/**
  ******************************************************************************
  * @file TIM/PWM_Input/main.c 
  * @author  MCD Application Team
  * @version  V3.0.0
  * @date  04/06/2009
  * @brief  Main program body
  ******************************************************************************
  * @copy
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2009 STMicroelectronics</center></h2>
  */ 

/* Includes ------------------------------------------------------------------*/
#include "stm32f10x.h"
#include "ascii16.h"
#include "logo.h"

/** @addtogroup StdPeriph_Examples
  * @{
  */

/** @addtogroup TIM_PWM_Input
  * @{
  */ 

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define LAT_LOW()       GPIO_ResetBits(GPIOA, GPIO_Pin_4)	//SPI1_NSS
#define LAT_HIGH()      GPIO_SetBits(GPIOA, GPIO_Pin_4)

#define BK_LOW()       GPIO_ResetBits(GPIOA, GPIO_Pin_6)	//PA6 (MISO None used)
#define BK_HIGH()      GPIO_SetBits(GPIOA, GPIO_Pin_6)

#define CLKG_LOW()       GPIO_ResetBits(GPIOA, GPIO_Pin_2)	//PA2
#define CLKG_HIGH()      GPIO_SetBits(GPIOA, GPIO_Pin_2)

#define SIG_LOW()       GPIO_ResetBits(GPIOA, GPIO_Pin_3)	//PA3
#define SIG_HIGH()      GPIO_SetBits(GPIOA, GPIO_Pin_3)

#define POS_X_MAX   127
#define POS_Y_MAX   7

#define SPI_MASTER                   SPI1
#define SPI_MASTER_CLK               RCC_APB2Periph_SPI1
#define SPI_MASTER_GPIO              GPIOA
#define SPI_MASTER_GPIO_CLK          RCC_APB2Periph_GPIOA  
#define SPI_MASTER_PIN_SCK           GPIO_Pin_5
#define SPI_MASTER_PIN_MISO          GPIO_Pin_6
#define SPI_MASTER_PIN_MOSI          GPIO_Pin_7
#define SPI_MASTER_DMA               DMA1
#define SPI_MASTER_DMA_CLK           RCC_AHBPeriph_DMA1  
#define SPI_MASTER_Rx_DMA_Channel    DMA1_Channel2
#define SPI_MASTER_Rx_DMA_FLAG       DMA1_FLAG_TC2
#define SPI_MASTER_Tx_DMA_Channel    DMA1_Channel3
#define SPI_MASTER_Tx_DMA_FLAG       DMA1_FLAG_TC3  
#define SPI_MASTER_DR_Base           0x4001300C
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
volatile unsigned long nTime = 0;
volatile unsigned char a_buf[8][128];
volatile unsigned char SPI_MASTER_Buffer_Tx[48];
volatile unsigned char pos_x = 0;
volatile unsigned char pos_y = 0;


/* Private function prototypes -----------------------------------------------*/
void RCC_Configuration(void);
void GPIO_Configuration(void);
void SPI1_SendByte(uint8_t byte);
void SPI2_SendByte_soft(uint8_t byte);
void Scan_VFD(void);



/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Main program
  * @param  None
  * @retval : None
  */
void delay_us(unsigned long us)
{
	unsigned char i = 0;
    while(us > 0)
    {
        us--;
        for(i = 0;i < 5;i++)
        {
        }
    }
}

void spi_init(void)
{
    SPI_InitTypeDef  SPI_InitStructure;
    DMA_InitTypeDef    DMA_InitStructure;

    SPI_InitStructure.SPI_Direction = SPI_Direction_1Line_Tx;
    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_High;
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_2Edge;
    SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_32; //72 / 16 = 4.5MHz
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_LSB;
    SPI_InitStructure.SPI_CRCPolynomial = 7;
    SPI_Init(SPI_MASTER, &SPI_InitStructure);

    /* SPI1_Tx_DMA_Channel configuration ---------------------------------*/
    DMA_DeInit(SPI_MASTER_Tx_DMA_Channel);  
    DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)SPI_MASTER_DR_Base;
    DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)SPI_MASTER_Buffer_Tx;
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
    DMA_InitStructure.DMA_Priority = DMA_Priority_Low;
    DMA_Init(SPI_MASTER_Tx_DMA_Channel, &DMA_InitStructure);

    /* Enable SPI_MASTER DMA Tx request */
    SPI_I2S_DMACmd(SPI_MASTER, SPI_I2S_DMAReq_Tx, ENABLE);

    

    /* Enable SPI_MASTER */
    SPI_Cmd(SPI_MASTER, ENABLE);

    /* Enable DMA channels */
    DMA_Cmd(SPI_MASTER_Tx_DMA_Channel, ENABLE);

    
    while(!DMA_GetFlagStatus(SPI_MASTER_Tx_DMA_FLAG));
    
    
}

void SPI1_SendByte(uint8_t byte)
{
    /*!< Loop while DR register in not emplty */
    while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE) == RESET);
    
    /*!< Send byte through the SPI1 peripheral */
    SPI_I2S_SendData(SPI1, byte);
}

void SPI2_SendByte_soft(uint8_t byte)
{
    unsigned char i = 0;
    for(i = 0;i < 5;i++)
    {
        CLKG_LOW();
        if((byte & 0x01) == 0x00)
        {
            SIG_LOW();
        }
        else
        {
            SIG_HIGH();
        }
        byte >>= 1;
        CLKG_HIGH();
    }
}

void Scan_VFD(void)
{
    static unsigned char gird = 43;
    unsigned char gird_tmp = 0;
    unsigned char tmp = 0;
    unsigned char i = 0;
    unsigned char base_a = 0;

    while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE) == RESET);

    //delay_us(1);

    BK_HIGH();
	delay_us(2);
	LAT_HIGH();
	delay_us(2);
	LAT_LOW();
    delay_us(1);
	BK_LOW();

    gird++;
    if(gird >= 43)
    {
        gird = 0;
        SPI2_SendByte_soft(0x03);
    }
    CLKG_LOW();
    delay_us(0);
    CLKG_HIGH();

    gird_tmp = 42 - gird;

    base_a = (gird_tmp >> 1) * 6;

    if((gird_tmp & 0x01) == 0x01)                       //˫
    {
        for(i = 0;i < 8;i++)
        {
            tmp = 0;
            
            if((a_buf[i][base_a + 5] & 0x01) == 0x01)
            {
                tmp = 0x02;
            }
            if((a_buf[i][base_a + 4] & 0x01) == 0x01)
            {
                tmp |= 0x08;
            }
            if((a_buf[i][base_a + 3] & 0x01) == 0x01)
            {
                tmp |= 0x20;
            }
            if((a_buf[i][base_a + 5] & 0x02) == 0x02)
            {
                tmp |= 0x80;
            }

            SPI1_SendByte(tmp);

            tmp  = a_buf[i][base_a + 4] & 0x02;
            if((a_buf[i][base_a + 3] & 0x02) == 0x02)
            {
                tmp |= 0x08;
            }
            if((a_buf[i][base_a + 5] & 0x04) == 0x04)
            {
                tmp |= 0x20;
            }
            if((a_buf[i][base_a + 4] & 0x04) == 0x04)
            {
                tmp |= 0x80;
            }

            SPI1_SendByte(tmp);

            tmp = 0;
            if((a_buf[i][base_a + 3] & 0x04) == 0x04)
            {
                tmp = 0x02;
            }
            tmp |= a_buf[i][base_a + 5] & 0x08;

            if((a_buf[i][base_a + 4] & 0x08) == 0x08)
            {
                tmp |= 0x20;
            }
            if((a_buf[i][base_a + 3] & 0x08) == 0x08)
            {
                tmp |= 0x80;
            }
            SPI1_SendByte(tmp);

            tmp = 0;
            if((a_buf[i][base_a + 5] & 0x10) == 0x10)
            {
                tmp = 0x02;
            }
            if((a_buf[i][base_a + 4] & 0x10) == 0x10)
            {
                tmp |= 0x08;
            }
            if((a_buf[i][base_a + 3] & 0x10) == 0x10)
            {
                tmp |= 0x20;
            }
            if((a_buf[i][base_a + 5] & 0x20) == 0x20)
            {
                tmp |= 0x80;
            }

            SPI1_SendByte(tmp);

            tmp = 0;
            if((a_buf[i][base_a + 4] & 0x20) == 0x20)
            {
                tmp = 0x02;
            }
            if((a_buf[i][base_a + 3] & 0x20) == 0x20)
            {
                tmp |= 0x08;
            }
            if((a_buf[i][base_a + 5] & 0x40) == 0x40)
            {
                tmp |= 0x20;
            }
            if((a_buf[i][base_a + 4] & 0x40) == 0x40)
            {
                tmp |= 0x80;
            }
            
            SPI1_SendByte(tmp);

            tmp = 0;
            if((a_buf[i][base_a + 3] & 0x40) == 0x40)
            {
                tmp = 0x02;
            }
            if((a_buf[i][base_a + 5] & 0x80) == 0x80)
            {
                tmp |= 0x08;
            }
            if((a_buf[i][base_a + 4] & 0x80) == 0x80)
            {
                tmp |= 0x20;
            }

            tmp |= a_buf[i][base_a + 3] & 0x80;
            
            SPI1_SendByte(tmp);
        }
    }
    else                                                //
    {
        for(i = 0;i < 8;i++)
        {
            tmp  = a_buf[i][base_a + 0] & 0x01;
            if((a_buf[i][base_a + 1] & 0x01) == 0x01)
            {
                tmp |= 0x04;
            }
            if((a_buf[i][base_a + 2] & 0x01) == 0x01)
            {
                tmp |= 0x10;
            }
            if((a_buf[i][base_a + 0] & 0x02) == 0x02)
            {
                tmp |= 0x40;
            }

            SPI1_SendByte(tmp);

            tmp = 0;
            if((a_buf[i][base_a + 1] & 0x02) == 0x02)
            {
                tmp = 0x01;
            }
            if((a_buf[i][base_a + 2] & 0x02) == 0x02)
            {
                tmp |= 0x04;
            }
            if((a_buf[i][base_a + 0] & 0x04) == 0x04)
            {
                tmp |= 0x10;
            }
            if((a_buf[i][base_a + 1] & 0x04) == 0x04)
            {
                tmp |= 0x40;
            }

            SPI1_SendByte(tmp);

            tmp = 0;
            if((a_buf[i][base_a + 2] & 0x04) == 0x04)
            {
                tmp = 0x01;
            }
            if((a_buf[i][base_a + 0] & 0x08) == 0x08)
            {
                tmp |= 0x04;
            }
            if((a_buf[i][base_a + 1] & 0x08) == 0x08)
            {
                tmp |= 0x10;
            }
            if((a_buf[i][base_a + 2] & 0x08) == 0x08)
            {
                tmp |= 0x40;
            }
            SPI1_SendByte(tmp);

            tmp = 0;
            if((a_buf[i][base_a + 0] & 0x10) == 0x10)
            {
                tmp = 0x01;
            }
            if((a_buf[i][base_a + 1] & 0x10) == 0x10)
            {
                tmp |= 0x04;
            }
            tmp |= a_buf[i][base_a + 2] & 0x10;
            if((a_buf[i][base_a + 0] & 0x20) == 0x20)
            {
                tmp |= 0x40;
            }

            SPI1_SendByte(tmp);

            tmp = 0;
            if((a_buf[i][base_a + 1] & 0x20) == 0x20)
            {
                tmp = 0x01;
            }
            if((a_buf[i][base_a + 2] & 0x20) == 0x20)
            {
                tmp |= 0x04;
            }
            if((a_buf[i][base_a + 0] & 0x40) == 0x40)
            {
                tmp |= 0x10;
            }
            tmp |= a_buf[i][base_a + 1] & 0x40;

            SPI1_SendByte(tmp);

            tmp = 0;
            if((a_buf[i][base_a + 2] & 0x40) == 0x40)
            {
                tmp = 0x01;
            }
            if((a_buf[i][base_a + 0] & 0x80) == 0x80)
            {
                tmp |= 0x04;
            }
            if((a_buf[i][base_a + 1] & 0x80) == 0x80)
            {
                tmp |= 0x10;
            }
            if((a_buf[i][base_a + 2] & 0x80) == 0x80)
            {
                tmp |= 0x40;
            }
            SPI1_SendByte(tmp);            
        }
    }
}

void vfd_putc_asc16(unsigned char x,unsigned char y,unsigned char ch)
{
    const unsigned char *pt1,*pt2;
    unsigned char i = 0;

    if((ch < 128) && (ch > 31) && (x < 16) && (y < 4))
    {
        pt1 = &ascii16x16[0] + ((ch - 32) * 16);
        pt2 = pt1 + 8;
        for(i = 0;i < 8;i++)
        {
            a_buf[y * 2][x * 8 + i] = *pt1;
            pt1++;
            a_buf[y *2  + 1][x * 8 + i] = *pt2;
            pt2++;
        }
    }
}

void vfd_puts_asc16(unsigned char x,unsigned char y,unsigned char *str)
{
    while(*str != 0x00)
    {
        vfd_putc_asc16(x,y,*str);
        str++;
        x++;
    }
}

void disp_image_64(unsigned char pos,const unsigned char *dat)
{
    unsigned char i = 0;
    unsigned char j = 0;

    for(i = 0;i < 8;i++)
    {
        for(j = 0;j < 64;j++)
        {
            a_buf[i][pos * 64 + j] = *dat;
            dat++;
        }
    }
}


int main(void)
{
	unsigned char i = 0;
    unsigned char j = 0;
    unsigned char test = 0;
    
    RCC_Configuration();
    spi_init();
    SysTick_Config(720000);     //10ms
    
    /* Configure the GPIO ports */
    GPIO_Configuration();
    
    for(j = 0;j < 8;j++)
    {
        for(i = 0;i < 128;i++)
        {
            a_buf[j][i] = 0x00;
        }
    }

    //vfd_putc_asc16(0,0,'A');
    //vfd_putc_asc16(0,1,'B');

    test = 0;
    while (1)
    {
         Scan_VFD();
         
         
         if(nTime == 0)
         {
            test++;
            if(test > 7)
            {
                test = 0;
            }

         
            switch (test)
            {
                case 1: //ALL ON
                    for(j = 0;j < 8;j++)
                    {
                        for(i = 0;i < 128;i++)
                        {
                            a_buf[j][i] = 0x00;
                        }
                    }
                    nTime = 100;
                    break;
                case 2: //ALL OFF
                    for(j = 0;j < 8;j++)
                    {
                        for(i = 0;i < 128;i++)
                        {
                            a_buf[j][i] = 0xFF;
                        }
                    }
                    nTime = 500;
                    break;
                case 3: //ALL ON
                    for(j = 0;j < 8;j++)
                    {
                        for(i = 0;i < 128;i++)
                        {
                            a_buf[j][i] = 0x00;
                        }
                    }

                    vfd_puts_asc16(0,0,"OURDEV VFD TEST.");
                    vfd_puts_asc16(0,1,"Debugging......");
                    vfd_puts_asc16(0,2,"2011-12-19 PM");
                    vfd_puts_asc16(0,3,"Thanks kazuyuki!");
                    nTime = 500;
                    break;
                case 4:
                    disp_image_64(0, &chifan[0]);
                    disp_image_64(1, &huoche[0]);
                    nTime = 500;
                    break;
                case 5:
                    disp_image_64(0, &hua[0]);
                    disp_image_64(1, &wu[0]);
                    nTime = 500;
                    break;
                case 6:
                    disp_image_64(0, &ball1[0]);
                    disp_image_64(1, &ball2[0]);
                    nTime = 500;
                    break;
                case 7:
                    disp_image_64(0, &lf[0]);
                    disp_image_64(1, &xx[0]);
                    nTime = 500;
                    break;
                default:
                    test = 0;
                    break;
            }
           
         }
         
    }  
}

/**
  * @brief  Configures the different system clocks.
  * @param  None
  * @retval : None
  */
void RCC_Configuration(void)
{
    /* Setup the microcontroller system. Initialize the Embedded Flash Interface,  
    initialize the PLL and update the SystemFrequency variable. */
    SystemInit();
    
    /* Periph clock enable */
    RCC_AHBPeriphClockCmd(SPI_MASTER_DMA_CLK, ENABLE);

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_SPI1 | RCC_APB2Periph_AFIO, ENABLE);
}

/**
  * @brief  Configure the GPIOD Pins.
  * @param  None
  * @retval : None
  */
void GPIO_Configuration(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    
    /* GPIOB configuration: PB5 PB6 PB7 PB8 as led controller */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5|GPIO_Pin_6|GPIO_Pin_7|GPIO_Pin_8;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
    
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    
    /*!< Configure SPI1 pins: SCK -- PA5 >*/
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    /*!< Configure SPI1 pins: MOSI -- PA7 >*/
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    /*!< Configure SPI1/2 comm pins: LAT A/G -- PA4 >*/
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

	/*!< Configure SPI1/2 comm pins: BK A/G -- PA6 >*/
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

	/*!< Configure SPI2(Software) pins: CLKG -- PA2 >*/
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    /*!< Configure SPI2(Software) pins: SIG -- PA3 >*/
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
}


#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
  */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  while (1)
  {}
}
#endif

/**
  * @}
  */ 

/**
  * @}
  */ 

/******************* (C) COPYRIGHT 2009 STMicroelectronics *****END OF FILE****/
